@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+
 # Copyright 2004-2018 Davide Alberani <da@erlug.linux.it>
 #           2008-2018 H. Turgut Uyar <uyar@tekir.org>
 #
@@ -31,22 +33,28 @@ plot summary
 ...and so on.
 """
 
+from __future__ import absolute_import, division, print_function, unicode_literals
+
 import functools
 import re
-import urllib.error
-import urllib.parse
-import urllib.request
 
+from imdb import PY2
 from imdb import imdbURL_base
 from imdb.Company import Company
 from imdb.Movie import Movie
 from imdb.Person import Person
 from imdb.utils import _Container, KIND_MAP
 
-from .piculet import Path, Rule, Rules, preprocessors
+from .piculet import Path, Rule, Rules, preprocessors, transformers
 from .utils import DOMParserBase, analyze_imdbid, build_person
 
 
+if PY2:
+    from urllib import unquote
+else:
+    from urllib.parse import unquote
+
+
 # Dictionary used to convert some section's names.
 _SECT_CONV = {
     'directed': 'director',
@@ -64,6 +72,7 @@ _SECT_CONV = {
     'production management': 'production manager',
     'second unit director or assistant director': 'assistant director',
     'costume and wardrobe department': 'costume department',
+    'costume departmen': 'costume department',
     'sound department': 'sound crew',
     'stunts': 'stunt performer',
     'other crew': 'miscellaneous crew',
@@ -227,8 +236,7 @@ def analyze_certificates(certificates):
 
 
 class DOMHTMLMovieParser(DOMParserBase):
-    """Parser for the "combined details" (and if instance.mdparse is
-    True also for the "main details") page of a given movie.
+    """Parser for the "reference" page of a given movie.
     The page should be provided as a string, as taken from
     the www.imdb.com server.  The final result will be a
     dictionary, with a key for every relevant section.
@@ -236,7 +244,7 @@ class DOMHTMLMovieParser(DOMParserBase):
     Example::
 
         mparser = DOMHTMLMovieParser()
-        result = mparser.parse(combined_details_html_string)
+        result = mparser.parse(reference_html_string)
     """
     _containsObjects = True
 
@@ -254,7 +262,7 @@ class DOMHTMLMovieParser(DOMParserBase):
                 foreach='//h4[contains(@class, "ipl-header__content")]',
                 rules=[
                     Rule(
-                        key=Path('./@name', transform=lambda x: x.replace('_', ' ')),
+                        key=Path('./@name', transform=lambda x: x.replace('_', ' ').strip()),
                         extractor=Rules(
                             foreach='../../following-sibling::table[1]//tr',
                             rules=[
@@ -367,7 +375,7 @@ class DOMHTMLMovieParser(DOMParserBase):
             key='aspect ratio',
             extractor=Path(
                 '//td[starts-with(text(), "Aspect")]/..//li/text()',
-                transform=str.strip
+                transform=transformers.strip
             )
         ),
         Rule(
@@ -477,7 +485,7 @@ class DOMHTMLMovieParser(DOMParserBase):
             key='season/episode',
             extractor=Path(
                 '//div[@class="titlereference-overview-season-episode-section"]/ul//text()',
-                transform=str.strip
+                transform=transformers.strip
             )
         ),
         Rule(
@@ -546,7 +554,7 @@ class DOMHTMLMovieParser(DOMParserBase):
             extractor=Path(
                 '//td[starts-with(text(), "Status Updated:")]/'
                 '..//div[@class="info-content"]//text()',
-                transform=str.strip
+                transform=transformers.strip
             )
         ),
         Rule(
@@ -554,7 +562,7 @@ class DOMHTMLMovieParser(DOMParserBase):
             extractor=Path(
                 '//td[starts-with(text(), "Comments:")]/'
                 '..//div[@class="info-content"]//text()',
-                transform=str.strip
+                transform=transformers.strip
             )
         ),
         Rule(
@@ -562,18 +570,18 @@ class DOMHTMLMovieParser(DOMParserBase):
             extractor=Path(
                 '//td[starts-with(text(), "Note:")]/'
                 '..//div[@class="info-content"]//text()',
-                transform=str.strip
+                transform=transformers.strip
             )
         ),
         Rule(
-            key='blackcatheader',
+            key='companies',
             extractor=Rules(
-                foreach='//b[@class="blackcatheader"]',
+                foreach="//ul[@class='simpleList']",
                 rules=[
                     Rule(
-                        key=Path('./text()', transform=str.lower),
+                        key=Path('preceding-sibling::header[1]/div/h4/text()', transform=transformers.lower),
                         extractor=Rules(
-                            foreach='../ul/li',
+                            foreach='./li',
                             rules=[
                                 Rule(
                                     key='name',
@@ -590,6 +598,7 @@ class DOMHTMLMovieParser(DOMParserBase):
                             ],
                             transform=lambda x: Company(
                                 name=x.get('name') or '',
+                                accessSystem='http',
                                 companyID=analyze_imdbid(x.get('comp-link')),
                                 notes=(x.get('notes') or '').strip()
                             )
@@ -692,7 +701,6 @@ class DOMHTMLMovieParser(DOMParserBase):
                                 for x in data['runtimes']]
         if 'number of seasons' in data:
             data['seasons'] = [str(i) for i in range(1, data['number of seasons'] + 1)]
-            # data['number of seasons'] = seasons[-1] if seasons else len(data['seasons'])
         if 'season/episode' in data:
             tokens = data['season/episode'].split('Episode')
             try:
@@ -704,30 +712,6 @@ class DOMHTMLMovieParser(DOMParserBase):
             except:
                 data['episode'] = 'unknown'
             del data['season/episode']
-        # if 'original air date' in data:
-        #     oid = self.re_space.sub(' ', data['original air date']).strip()
-        #     data['original air date'] = oid
-        #     aid = self.re_airdate.findall(oid)
-        #     if aid and len(aid[0]) == 3:
-        #         date, season, episode = aid[0]
-        #         date = date.strip()
-        #         try:
-        #             season = int(season)
-        #         except ValueError:
-        #             pass
-        #         try:
-        #             episode = int(episode)
-        #         except ValueError:
-        #             pass
-        #         if date and date != '????':
-        #             data['original air date'] = date
-        #         else:
-        #             del data['original air date']
-        #         # Handle also "episode 0".
-        #         if season or isinstance(season, int):
-        #             data['season'] = season
-        #         if episode or isinstance(season, int):
-        #             data['episode'] = episode
         for k in ('writer', 'director'):
             t_k = 'thin %s' % k
             if t_k not in data:
@@ -770,6 +754,14 @@ class DOMHTMLMovieParser(DOMParserBase):
                 data['votes'] = int(votes)
             except (TypeError, ValueError):
                 pass
+        companies = data.get('companies')
+        if companies:
+            for section in companies:
+                for key, value in section.items():
+                    if key in data:
+                        key = '%s companies' % key
+                    data.update({key: value})
+            del data['companies']
         return data
 
 
@@ -1091,7 +1083,7 @@ class DOMHTMLAlternateVersionsParser(DOMParserBase):
             extractor=Path(
                 foreach='//ul[@class="trivia"]/li',
                 path='.//text()',
-                transform=str.strip
+                transform=transformers.strip
             )
         )
     ]
@@ -1116,7 +1108,7 @@ class DOMHTMLTriviaParser(DOMParserBase):
             extractor=Path(
                 foreach='//div[@class="sodatext"]',
                 path='.//text()',
-                transform=str.strip
+                transform=transformers.strip
             )
         )
     ]
@@ -1148,7 +1140,7 @@ class DOMHTMLSoundtrackParser(DOMParserBase):
             extractor=Path(
                 foreach='//div[@class="list"]//div',
                 path='.//text()',
-                transform=str.strip
+                transform=transformers.strip
             )
         )
     ]
@@ -1362,11 +1354,11 @@ class DOMHTMLReleaseinfoParser(DOMParserBase):
                 foreach='//table[@id="akas"]//tr',
                 rules=[
                     Rule(
-                        key='title',
+                        key='countries',
                         extractor=Path('./td[1]/text()')
                     ),
                     Rule(
-                        key='countries',
+                        key='title',
                         extractor=Path('./td[2]/text()')
                     )
                 ]
@@ -1393,12 +1385,15 @@ class DOMHTMLReleaseinfoParser(DOMParserBase):
             date = date.strip()
             if not (country and date):
                 continue
-            notes = i['notes']
+            notes = i.get('notes')
             info = '%s::%s' % (country, date)
             if notes:
+                notes = notes.replace('\n', '')
+                i['notes'] = notes
                 info += notes
             rl.append(info)
         if releases:
+            data['raw release dates'] = data['release dates']
             del data['release dates']
         if rl:
             data['release dates'] = rl
@@ -1415,9 +1410,10 @@ class DOMHTMLReleaseinfoParser(DOMParserBase):
                 for country in countries:
                     nakas.append('%s::%s' % (title, country.strip()))
         if akas:
+            data['raw akas'] = data['akas']
             del data['akas']
         if nakas:
-            data['akas from release info'] = nakas
+            data['akas'] = data['akas from release info'] = nakas
         return data
 
 
@@ -1730,7 +1726,7 @@ class DOMHTMLOfficialsitesParser(DOMParserBase):
                 ],
                 transform=lambda x: (
                     x.get('info').strip(),
-                    urllib.parse.unquote(_normalize_href(x.get('link')))
+                    unquote(_normalize_href(x.get('link')))
                 )
             )
         )
@@ -1757,7 +1753,7 @@ class DOMHTMLConnectionParser(DOMParserBase):
                 foreach='//div[@class="_imdbpy"]',
                 rules=[
                     Rule(
-                        key=Path('./h5/text()', transform=str.lower),
+                        key=Path('./h5/text()', transform=transformers.lower),
                         extractor=Rules(
                             foreach='./a',
                             rules=[
@@ -1886,26 +1882,15 @@ class DOMHTMLTechParser(DOMParserBase):
     ]
 
     def postprocess_data(self, data):
-        for key in data:
-            data[key] = [x for x in data[key] if x != '|']
-            data[key] = [self.re_space.sub(' ', x).strip() for x in data[key]]
-            data[key] = [_f for _f in data[key] if _f]
-        if self.kind == 'contacts' and data:
-            data = {self.kind: data}
-        else:
-            if self.kind == 'publicity':
-                if 'biography (print)' in data:
-                    data['biography-print'] = data['biography (print)']
-                    del data['biography (print)']
-            # Tech info.
-            for key in list(data.keys()):
-                if key.startswith('film negative format'):
-                    data['film negative format'] = data[key]
-                    del data[key]
-                elif key.startswith('film length'):
-                    data['film length'] = data[key]
-                    del data[key]
-        return data
+        info = {}
+        for section in data.get('tech', []):
+            info.update(section)
+        for key, value in info.items():
+            if isinstance(value, list):
+                info[key] = [self.re_space.sub(' ', x).strip() for x in value]
+            else:
+                info[key] = self.re_space.sub(' ', value).strip()
+        return {self.kind: info}
 
 
 class DOMHTMLNewsParser(DOMParserBase):
@@ -2026,7 +2011,7 @@ class DOMHTMLSeasonEpisodesParser(DOMParserBase):
             key='_seasons',
             extractor=Path(
                 foreach='//select[@id="bySeason"]//option',
-                path='/@value'
+                path='./@value'
             )
         ),
         Rule(
@@ -2274,7 +2259,7 @@ class DOMHTMLEpisodesParser(DOMParserBase):
                         foreach='//h4',
                         rules=[
                             Rule(
-                                key=Path('./text()[1]', transform=str.strip),
+                                key=Path('./text()[1]', transform=transformers.strip),
                                 extractor=Rules(
                                     foreach='./following-sibling::table[1]//td[@class="nm"]',
                                     rules=[
@@ -2528,7 +2513,7 @@ class DOMHTMLParentsGuideParser(DOMParserBase):
                     Rule(
                         key=Path(
                             './h3/a/span/text()',
-                            transform=str.lower
+                            transform=transformers.lower
                         ),
                         extractor=Path(
                             foreach='../following-sibling::div[1]/p',
