@@ -30,6 +30,8 @@ biography
 ...and so on.
 """
 
+from __future__ import absolute_import, division, print_function, unicode_literals
+
 import re
 
 from imdb.utils import analyze_name, canonicalName
@@ -40,7 +42,7 @@ from .movieParser import (
     DOMHTMLOfficialsitesParser,
     DOMHTMLTechParser
 )
-from .piculet import Path, Rule, Rules
+from .piculet import Path, Rule, Rules, transformers
 from .utils import DOMParserBase, analyze_imdbid, build_movie
 
 
@@ -264,7 +266,10 @@ class DOMHTMLBioParser(DOMParserBase):
     _birth_rules = [
         Rule(
             key='birth date',
-            extractor=Path('./time[@itemprop="birthDate"]/@datetime')
+            extractor=Path(
+                './time/@datetime',
+                transform=lambda s: '%4d-%02d-%02d' % tuple(map(int, s.split('-')))
+            )
         ),
         Rule(
             key='birth notes',
@@ -275,7 +280,10 @@ class DOMHTMLBioParser(DOMParserBase):
     _death_rules = [
         Rule(
             key='death date',
-            extractor=Path('./time[@itemprop="deathDate"]/@datetime')
+            extractor=Path(
+                './time/@datetime',
+                transform=lambda s: '%4d-%02d-%02d' % tuple(map(int, s.split('-')))
+            )
         ),
         Rule(
             key='death cause',
@@ -339,14 +347,13 @@ class DOMHTMLBioParser(DOMParserBase):
             extractor=Path(
                 '//table[@id="overviewTable"]'
                 '//td[text()="Height"]/following-sibling::td[1]/text()',
-                transform=str.strip
+                transform=transformers.strip
             )
         ),
         Rule(
             key='mini biography',
             extractor=Rules(
-                foreach='//a[@name="mini_bio"]/following-sibling::'
-                        'div[1 = count(preceding-sibling::a[1] | ../a[@name="mini_bio"])]',
+                foreach='//h4[starts-with(text(), "Mini Bio")]/following-sibling::div',
                 rules=[
                     Rule(
                         key='bio',
@@ -388,7 +395,7 @@ class DOMHTMLBioParser(DOMParserBase):
                 foreach='//div[@class="_imdbpyh4"]/h4[starts-with(text(), "Trade Mark")]'
                         '/.././div[contains(@class, "soda")]',
                 path='.//text()',
-                transform=str.strip
+                transform=transformers.strip
             )
         ),
         Rule(
@@ -397,7 +404,7 @@ class DOMHTMLBioParser(DOMParserBase):
                 foreach='//div[@class="_imdbpyh4"]/h4[starts-with(text(), "Trivia")]'
                         '/.././div[contains(@class, "soda")]',
                 path='.//text()',
-                transform=str.strip
+                transform=transformers.strip
             )
         ),
         Rule(
@@ -406,7 +413,7 @@ class DOMHTMLBioParser(DOMParserBase):
                 foreach='//div[@class="_imdbpyh4"]/h4[starts-with(text(), "Personal Quotes")]'
                         '/.././div[contains(@class, "soda")]',
                 path='.//text()',
-                transform=str.strip
+                transform=transformers.strip
             )
         ),
         Rule(
@@ -469,7 +476,7 @@ class DOMHTMLOtherWorksParser(DOMParserBase):
             extractor=Path(
                 foreach='//li[@class="ipl-zebra-list__item"]',
                 path='.//text()',
-                transform=str.strip
+                transform=transformers.strip
             )
         )
     ]
