@@ -20,16 +20,19 @@ This module provides miscellaneous utilities used by the components
 in the :mod:`imdb.parser.http` package.
 """
 
+from __future__ import absolute_import, division, print_function, unicode_literals
+
 import collections
 import logging
 import re
-from html.entities import entitydefs
 
+from imdb import PY2
+from imdb.Character import Character
 from imdb.Movie import Movie
 from imdb.Person import Person
 from imdb.utils import _Container, flatten
 
-from .piculet import ElementTree, Rules, build_tree
+from .piculet import _USE_LXML, ElementTree, Rules, build_tree, html_to_xhtml
 from .piculet import xpath as piculet_xpath
 from .piculet import Rule, Path
 
@@ -82,37 +85,6 @@ def _putRefs(d, re_titles, re_names, lastKey=None):
                 _putRefs(d[k], re_titles, re_names, lastKey=lastKey)
 
 
-# Handle HTML/XML/SGML entities.
-entitydefs = entitydefs.copy()
-entitydefsget = entitydefs.get
-entitydefs['nbsp'] = ' '
-
-sgmlentity = {'lt': '<', 'gt': '>', 'amp': '&', 'quot': '"', 'apos': '\'', 'ndash': '-'}
-_sgmlentkeys = list(sgmlentity.keys())
-
-entcharrefs = {}
-entcharrefsget = entcharrefs.get
-for _k, _v in list(entitydefs.items()):
-    if _k in _sgmlentkeys:
-        continue
-    if _v[0:2] == '&#':
-        dec_code = _v[1:-1]
-        _v = chr(int(_v[2:-1]))
-        entcharrefs[dec_code] = _v
-    else:
-        dec_code = '#' + str(ord(_v))
-        entcharrefs[dec_code] = _v
-    entcharrefs[_k] = _v
-del _sgmlentkeys, _k, _v
-
-re_entcharrefs = re.compile('&(%s|\#160|\#\d{1,5}|\#x[0-9a-f]{1,4});' %
-                            '|'.join(map(re.escape, entcharrefs)), re.I)
-re_entcharrefssub = re_entcharrefs.sub
-
-sgmlentity.update(dict([('#34', '"'), ('#38', '&'),
-                        ('#60', '<'), ('#62', '>'), ('#39', "'")]))
-
-
 _b_p_logger = logging.getLogger('imdbpy.parser.http.build_person')
 
 
@@ -222,7 +194,8 @@ def build_person(txt, personID=None, billingPos=None,
         for idx, role in enumerate(person.currentRole):
             if roleNotes[idx]:
                 role.notes = roleNotes[idx]
-    elif person.currentRole and not person.currentRole.notes and notes:
+    elif person.currentRole and isinstance(person.currentRole, Character) and \
+            not person.currentRole.notes and notes:
         person.currentRole.notes = notes
     return person
 
@@ -434,12 +407,12 @@ class DOMParserBase(object):
             self.getRefs = getRefs
         else:
             self.getRefs = self._defGetRefs
-        # TODO: get rid of the special entity handling
-        html_string = html_string.replace('&nbsp;', ' ')
+        if PY2 and isinstance(html_string, str):
+            html_string = html_string.decode('utf-8')
         # Temporary fix: self.parse_dom must work even for empty strings.
         html_string = self.preprocess_string(html_string)
-        html_string = html_string.strip()
         if html_string:
+            html_string = html_string.replace('&nbsp;', ' ')
             dom = self.get_dom(html_string)
             try:
                 dom = self.preprocess_dom(dom)
@@ -468,6 +441,8 @@ class DOMParserBase(object):
     def get_dom(self, html_string):
         """Return a dom object, from the given string."""
         try:
+            if not _USE_LXML:
+                html_string = html_to_xhtml(html_string, omit_tags={"script"})
             dom = build_tree(html_string, force_html=True)
             if dom is None:
                 dom = build_tree('')
@@ -515,7 +490,7 @@ class DOMParserBase(object):
             # re._pattern_type is present only since Python 2.5.
             if isinstance(getattr(src, 'sub', None), collections.Callable):
                 html_string = src.sub(sub, html_string)
-            elif isinstance(src, str):
+            elif isinstance(src, str) or isinstance(src, unicode):
                 html_string = html_string.replace(src, sub)
             elif isinstance(src, collections.Callable):
                 try:
