@@ -19,9 +19,12 @@
 This module provides basic utilities for the imdb package.
 """
 
+from __future__ import absolute_import, division, print_function, unicode_literals
+
 import logging
 import re
 import string
+import sys
 from copy import copy, deepcopy
 from functools import total_ordering
 from time import strftime, strptime
@@ -31,6 +34,9 @@ from imdb import linguistics
 from imdb._exceptions import IMDbParserError
 
 
+PY2 = sys.hexversion < 0x3000000
+
+
 # Logger for imdb.utils module.
 _utils_logger = logging.getLogger('imdbpy.utils')
 
@@ -150,7 +156,7 @@ def analyze_name(name, canonical=None):
     raise an IMDbParserError exception if the name is not valid.
     """
     original_n = name
-    name = name.strip()
+    name = name.split(' aka ')[0].strip()
     res = {}
     imdbIndex = ''
     opi = name.rfind('(')
@@ -309,7 +315,7 @@ def analyze_title(title, canonical=None, canonicalSeries=None, canonicalEpisode=
         canonicalSeries = canonicalEpisode = canonical
     original_t = title
     result = {}
-    title = title.strip()
+    title = title.split(' aka ')[0].strip()
     year = ''
     kind = ''
     imdbIndex = ''
@@ -979,6 +985,7 @@ TAGS_TO_MODIFY = {
     'movie.parents-guide': ('item', True),
     'movie.number-of-votes': ('item', True),
     'movie.soundtrack.item': ('item', True),
+    'movie.soundtrack.item.item': ('item', True),
     'movie.quotes': ('quote', False),
     'movie.quotes.quote': ('line', False),
     'movie.demographic': ('item', True),
@@ -995,7 +1002,8 @@ TAGS_TO_MODIFY = {
 
 
 _valid_chars = string.ascii_lowercase + '-' + string.digits
-_translator = str.maketrans(_valid_chars, _valid_chars)
+_translator = str.maketrans(_valid_chars, _valid_chars) if not PY2 else \
+    string.maketrans(_valid_chars, _valid_chars)
 
 
 def _tagAttr(key, fullpath):
@@ -1013,7 +1021,7 @@ def _tagAttr(key, fullpath):
         attrs['keytype'] = strType
         tagName = str(key)
     else:
-        tagName = str((key, 'ascii', 'ignore'))
+        tagName = key
     if isinstance(key, int):
         attrs['keytype'] = 'int'
     origTagName = tagName
@@ -1094,7 +1102,7 @@ def _seq2xml(seq, _l=None, withRefs=False, modFunct=None,
     else:
         if isinstance(seq, _Container):
             _l.extend(_tag4TON(seq))
-        else:
+        elif seq:
             # Text, ints, floats and the like.
             _l.append(_normalizeValue(seq, withRefs=withRefs,
                                       modFunct=modFunct,
@@ -1206,13 +1214,19 @@ class _Container(object):
                 self.currentRole.characterID = roleID
             else:
                 for index, item in enumerate(roleID):
-                    self.__role[index].characterID = item
+                    r = self.__role[index]
+                    if PY2 and isinstance(r, unicode):
+                        continue
+                    r.characterID = item
         else:
             if not isinstance(roleID, (list, tuple)):
                 self.currentRole.personID = roleID
             else:
                 for index, item in enumerate(roleID):
-                    self.__role[index].personID = item
+                    r = self.__role[index]
+                    if PY2 and isinstance(r, unicode):
+                        continue
+                    r.personID = item
 
     roleID = property(_get_roleID, _set_roleID,
                       doc="the characterID or personID of the currentRole object.")
@@ -1258,7 +1272,15 @@ class _Container(object):
             return None
         url = self.data[self._image_key] or ''
         ext_idx = url.rfind('.')
-        return url[:url.rindex('@')+1] + url[ext_idx:]
+        if ext_idx == -1:
+            return url
+        if '@' in url:
+            return url[:url.rindex('@')+1] + url[ext_idx:]
+        else:
+            prev_dot = url[:ext_idx].rfind('.')
+            if prev_dot == -1:
+                return url
+            return url[:prev_dot] + url[ext_idx:]
 
     def reset(self):
         """Reset the object."""
